# VMXON

## Prerequisites

### Nomenclature

- In today's standards, a virtual processor (vCPU) (aka **logical processor**)
is equivalent to a physical core.

## VMXON Region

**Section 26.11.5 p. 4'073, 2024 Manual**

The VMXON region is a **per logical processor** data structure that's allocated
and will be used by the physical processor for any operations related to VMX.

- Typically aligned to 4KiB (4096 bytes) which essentially implies that the
lowest 12 bits (11:0) should be 0.
    - This could be implementation specific depending on the processor.
- The necessary amount of memory required for the VMXON region can be read
from the `IA32_VMX_BASIC` MSR ([Model-specific register](https://en.wikipedia.org/wiki/Model-specific_register)).
- The VMXON region should be **zeroed** prior to executing the `vmxon`
instruction.
    - The first 4 bytes (32 bits) of the VMXON region, will subsequently contain
    the VMCS revision identifier. They should be written to the beginning of
    the VMXON region before executing `vmxon`
- The operand to `vmxon` will be the **physical address** of the allocated
VMXON region (also called the **VMXON pointer**).

![VMXON Region Format](./figs/vmxon/vmxon_region_format.png)

## VMCS Region

- The VMCS (Virtual Machine Control Structure) region is also a **per logical
processor** data structure that's allocated and used by the physical processor
for VMX operations.

### Structure of a VMCS region

From what I gather, the `union` in the `struct` definition below is needed because
depeding on the documentation (e.g. specifically in the 2024), only the first
**31** out of **32** bits should contain the VMCS revision identifier and bit 31
in that **specific case** should be cleared to 0 (_shadow_vmcs_indicator_).
When reading other sources, they say that the first 4 bytes (all 32 bits) should
container the VMCS identifier.

The `struct` below should also be `__attribute__((packed))` in the case of `gcc`,
or `clang`. Because the example below was compiled for Windows (MSVC probably),
packing was probably enabled some other way.

```c
struct __vmcs_t
{
    union 
    {
        unsigned int all;
        struct
        {
            unsigned int revision_identifier : 31;
            unsigned int shadow_vmcs_indicator : 1;
        } bits;
    } header;
    unsigned int abort_indicator;
    char data[ 0x1000 - 2 * sizeof( unsigned ) ];
};
```

![VMCS Region Format](./figs/vmxon/vmcs_region_format.png) 

### Structure of a vCPU

Based on the article [Entering VMX Operation](https://revers.engineering/day-2-entering-vmx-operation/),
a vCPU is essentially comprised of two fields which are the physical addresses
of the VMXON and VMCS regions.

```c
struct __vcpu_t
{
    struct __vmcs_t *vmcs;
    unsigned __int64 vmcs_physical;

    struct __vmcs_t *vmxon;
    unsigned __int64 vmxon_physical;
};
```

#### Remarks

After having read a couple of times the above mentioned article, the re-use of
`struct __vmcs_t` for the VMXON and VMCS regions seems somewhat dubious,
especially in the case of the VMXON region. The Intel documentation states,
that the VMXON region should be:

- 4KiB aligned
- The first 31 out of 32 bits should contain the VMCS revision identifier
- Bit 31 should be cleared to 0
- Rest of the 4KiB should be zeroed

A `struct` could look something like this:

```c
struct vmxon_region {
    /* Bit 31 of the revision identifier should be cleared to 0! */
    unsigned int vmcs_rev_id;
    char data[0x1000 - sizeof(unsigned int)];
}__attribute__((packed));
```

## TL;DR

From what I gather, a logical processor (vCPU) is essentially represented by a
4KiB region in memory whose name is the **VMCS** (in Intel nomenclature or
**VMCB** in AMD's one).

The data in the VMCS is organized as follows:

- **Guest-state area**: Processor state is saved into the guest-state area on VM
exits and loaded from there on VM entries.
- **Host-state area**: Processor state is loaded from the host-state area on VM
exits.
- **VM-execution control fields**: These fields control processor behavior in VMX
non-root operation. They determine in part the causes of VM exits.
- **VM-exit control fields**: These fields control VM exits.
- **VM-entry control fields**: These fields control VM entries.
- **VM-exit information fields**: These fields receive information on VM exits and
describe the cause and the nature of VM exits. On some processors, these fields
are read-only.

## Sources

- [Entering VMX Operation](https://revers.engineering/day-2-entering-vmx-operation/)
- Intel64 IA32 Manual Vol 3. 2007
    - It can be found at the following [link](https://githepia.hesge.ch/flg_courses/advanced_systems_programming/advanced_systems_programming_pub_fall24/-/tree/master/resources/Intel_Manuals?ref_type=heads)
